@@ -13,11 +13,14 @@ from typing import (
     List,
 )
 
-import scipy.sparse
+import numpy as np
 import weaviate
+from weaviate.auth import AuthCredentials
 
 from ..base.backend import BaseBackendMixin
 from .... import Document
+from ....helper import dataclass_from_dict
+from ..registry import _REGISTRY
 
 if TYPE_CHECKING:
     from ....types import (
@@ -30,20 +33,22 @@ class WeaviateConfig:
     """This class stores the config variables to initialize
     connection to the Weaviate server"""
 
-    client: Optional[Union[str, weaviate.Client]] = None
-    n_dim: Optional[int] = None
+    host: Optional[str] = field(default='localhost')
+    port: Optional[int] = field(default=8080)
+    protocol: Optional[str] = field(default='http')
     name: Optional[str] = None
     serialize_config: Dict = field(default_factory=dict)
+    n_dim: Optional[int] = None  # deprecated, not used anymore since weaviate 1.10
 
 
 class BackendMixin(BaseBackendMixin):
-    """Provide necessary functions to enable this storage backend. """
+    """Provide necessary functions to enable this storage backend."""
 
     def _init_storage(
         self,
-        docs: Optional['DocumentArraySourceType'] = None,
-        config: Optional[WeaviateConfig] = None,
-        **kwargs
+        _docs: Optional['DocumentArraySourceType'] = None,
+        config: Optional[Union[WeaviateConfig, Dict]] = None,
+        **kwargs,
     ):
         """Initialize weaviate storage.
 
@@ -54,49 +59,48 @@ class BackendMixin(BaseBackendMixin):
             raise an error if both are provided
         """
 
-        from ... import DocumentArray
-
-        self._schemas = None
-
         if not config:
             config = WeaviateConfig()
+        elif isinstance(config, dict):
+            config = dataclass_from_dict(WeaviateConfig, config)
 
-        self.n_dim = config.n_dim or 1
-        self.serialize_config = config.serialize_config
+        from ... import DocumentArray
 
-        import weaviate
+        self._serialize_config = config.serialize_config
 
-        if config.client is None:
-            config.client = 'http://localhost:8080'
-
-        if isinstance(config.client, str):
-            self._client = weaviate.Client(config.client)
-        else:
-            self._client = config.client
-
-        if config.name is not None and docs is not None:
+        if config.name and config.name != config.name.capitalize():
             raise ValueError(
-                'only one of name or docs can be provided for initialization'
+                'Weaviate class name has to be capitalized. '
+                'Please capitalize when declaring the name field in config.'
             )
+
+        self._persist = bool(config.name)
+
+        self._client = weaviate.Client(
+            f'{config.protocol}://{config.host}:{config.port}'
+        )
         self._config = config
 
         self._schemas = self._load_or_create_weaviate_schema()
-        self._offset2ids, self._offset2ids_wid = self._get_offset2ids_meta()
 
-        if docs is None and config.name:
-            return
+        _REGISTRY[self.__class__.__name__][self._class_name].append(self)
+
+        super()._init_storage(_docs, **kwargs)
 
         # To align with Sqlite behavior; if `docs` is not `None` and table name
         # is provided, :class:`DocumentArraySqlite` will clear the existing
         # table and load the given `docs`
-        self.clear()
-        if isinstance(
-            docs, (DocumentArray, Sequence, Generator, Iterator, itertools.chain)
+        if _docs is None:
+            return
+        elif isinstance(
+            _docs, (DocumentArray, Sequence, Generator, Iterator, itertools.chain)
         ):
-            self.extend(docs)
+            self.clear()
+            self.extend(_docs)
         else:
-            if isinstance(docs, Document):
-                self.append(docs)
+            self.clear()
+            if isinstance(_docs, Document):
+                self.append(_docs)
 
     def _get_weaviate_class_name(self) -> str:
         """Generate the class/schema name using the ``uuid1`` module with some
@@ -122,9 +126,7 @@ class BackendMixin(BaseBackendMixin):
                 {
                     'class': cls_name,
                     "vectorizer": "none",
-                    # TODO: this skips checking embedding dimension but might not
-                    # work if want to leverage weaviate for vector search
-                    'vectorIndexConfig': {'skip': True},
+                    'vectorIndexConfig': {'skip': False},
                     'properties': [
                         {
                             'dataType': ['blob'],
@@ -150,12 +152,10 @@ class BackendMixin(BaseBackendMixin):
 
     def _load_or_create_weaviate_schema(self):
         """Create a new weaviate schema for this :class:`DocumentArrayWeaviate` object
-        if not present in weaviate or if ``cls_name`` not provided, else if ``cls_name`` is provided
-        load the object with the given ``cls_name``
+        if not present in weaviate or if ``self._config.name`` is None. If ``self._config.name``
+        is provided and not None and schema with the specified name exists in weaviate,
+        then load the object with the given ``self._config.name``
 
-        :param cls_name: if provided, load this :class:`DocumentArrayWeaviate` object with
-            the data stored in weaviate. If ``cls_name`` is ``None``, the create a schema in weaviate
-            with a newly generated class name.
         :return: the schemas of this :class`DocumentArrayWeaviate` object and its meta
         """
         if not self._config.name:
@@ -181,14 +181,14 @@ class BackendMixin(BaseBackendMixin):
             self._offset2ids_wid
         ):
             self._client.data_object.update(
-                data_object={'_offset2ids': self._offset2ids},
+                data_object={'_offset2ids': self._offset2ids.ids},
                 class_name=self._meta_name,
                 uuid=self._offset2ids_wid,
             )
         else:
             self._offset2ids_wid = str(uuid.uuid1())
             self._client.data_object.create(
-                data_object={'_offset2ids': self._offset2ids},
+                data_object={'_offset2ids': self._offset2ids.ids},
                 class_name=self._meta_name,
                 uuid=self._offset2ids_wid,
             )
@@ -277,23 +277,30 @@ class BackendMixin(BaseBackendMixin):
         :param value: document to create a payload for
         :return: the payload dictionary
         """
-        if value.embedding is None:
-            embedding = [0] * self.n_dim
-        elif isinstance(value.embedding, scipy.sparse.spmatrix):
-            embedding = value.embedding.toarray()
-        else:
+        if value.embedding is not None:
             from ....math.ndarray import to_numpy_array
 
             embedding = to_numpy_array(value.embedding)
 
+            if embedding.ndim > 1:
+                embedding = np.asarray(embedding).squeeze()
+
+            # Weaviate expects vector to have dim 2 at least
+            # or get weaviate.exceptions.UnexpectedStatusCodeException:  models.C11yVector
+            # hence we cast it to list of a single element
+            if len(embedding) == 1:
+                embedding = [embedding[0]]
+        else:
+            embedding = None
+
         return dict(
-            data_object={'_serialized': value.to_base64(**self.serialize_config)},
+            data_object={'_serialized': value.to_base64(**self._serialize_config)},
             class_name=self._class_name,
-            uuid=self.wmap(value.id),
+            uuid=self._wmap(value.id),
             vector=embedding,
         )
 
-    def wmap(self, doc_id: str):
+    def _wmap(self, doc_id: str):
         """the function maps doc id to weaviate id
 
         :param doc_id: id of the document
@@ -306,11 +313,22 @@ class BackendMixin(BaseBackendMixin):
         return str(uuid.uuid5(uuid.NAMESPACE_URL, doc_id + self._class_name))
 
     def _get_storage_infos(self) -> Dict:
-        storage_infos = super()._get_storage_infos()
         return {
-            'Backend': 'Weaviate (www.semi.technology/developers/weaviate)',
-            'Hostname': self._config.client,
+            'Backend': 'Weaviate',
+            'Host': self._config.host,
+            'Port': str(self._config.port),
+            'Protocol': self._config.protocol,
             'Schema Name': self._config.name,
             'Serialization Protocol': self._config.serialize_config.get('protocol'),
-            **storage_infos,
         }
+
+    def __getstate__(self):
+        d = dict(self.__dict__)
+        del d['_client']
+        return d
+
+    def __setstate__(self, state):
+        self.__dict__ = state
+        self._client = weaviate.Client(
+            f'{state["_config"].protocol}://{state["_config"].host}:{state["_config"].port}'
+        )
